Terminal

Struct Terminal 

pub struct Terminal<B>
where B: Backend,
{ pub viewport_area: Rect, pub last_known_screen_size: Size, pub last_known_cursor_pos: Position, /* private fields */ }

Fields

viewport_area: Rect

Area of the viewport

last_known_screen_size: Size

Last known size of the terminal. Used to detect if the internal buffers have to be resized.

last_known_cursor_pos: Position

Last known position of the cursor. Used to find the new area when the viewport is inlined and the terminal resized.

Implementations

Source

impl<B> Terminal<B>
where B: Backend,

pub fn with_options(backend: B) -> Result<Self>

Creates a new Terminal with the given Backend and [TerminalOptions].

Example
use std::io::stdout;

use ratatui::{backend::CrosstermBackend, layout::Rect, Terminal, TerminalOptions, Viewport};

let backend = CrosstermBackend::new(stdout());
let viewport = Viewport::Fixed(Rect::new(0, 0, 10, 10));
let terminal = Terminal::with_options(backend, TerminalOptions { viewport })?;
pub const fn get_frame(&mut self) -> Frame<'_>

Get a Frame object which provides a consistent view into the terminal state for rendering.

pub const fn current_buffer_mut(&mut self) -> &mut Buffer

Gets the current buffer as a mutable reference.

pub const fn backend(&self) -> &B

Gets the backend

pub const fn backend_mut(&mut self) -> &mut B

Gets the backend as a mutable reference

pub fn flush(&mut self) -> Result<()>

Obtains a difference between the previous and the current buffer and passes it to the current backend for drawing.

pub const fn resize(&mut self, screen_size: Size) -> Result<()>

Updates the Terminal so that internal buffers match the requested area.

Requested area will be saved to remain consistent when rendering. This leads to a full clear of the screen.

pub fn set_viewport_area(&mut self, area: Rect)

Sets the viewport area.

pub fn autoresize(&mut self) -> Result<()>

Queries the backend for size and resizes if it doesn’t match the previous size.

pub fn draw<F>(&mut self, render_callback: F) -> Result<()>
where F: FnOnce(&mut Frame<'_>),

Draws a single frame to the terminal.

Returns a [CompletedFrame] if successful, otherwise a std::io::Error.

If the render callback passed to this method can fail, use try_draw instead.

Applications should call draw or try_draw in a loop to continuously render the terminal. These methods are the main entry points for drawing to the terminal.

This method will:

  • autoresize the terminal if necessary
  • call the render callback, passing it a Frame reference to render to
  • flush the current internal state by copying the current buffer to the backend
  • move the cursor to the last known position if it was set during the rendering closure

The render callback should fully render the entire frame when called, including areas that are unchanged from the previous frame. This is because each frame is compared to the previous frame to determine what has changed, and only the changes are written to the terminal. If the render callback does not fully render the frame, the terminal will not be in a consistent state.

Examples
use ratatui::{layout::Position, widgets::Paragraph};

// with a closure
terminal.draw(|frame| {
    let area = frame.area();
    frame.render_widget(Paragraph::new("Hello World!"), area);
    frame.set_cursor_position(Position { x: 0, y: 0 });
})?;

// or with a function
terminal.draw(render)?;

fn render(frame: &mut ratatui::Frame) {
    frame.render_widget(Paragraph::new("Hello World!"), frame.area());
}
pub fn try_draw<F, E>(&mut self, render_callback: F) -> Result<()>
where F: FnOnce(&mut Frame<'_>) -> Result<(), E>, E: Into<Error>,

Tries to draw a single frame to the terminal.

Returns Result::Ok containing a [CompletedFrame] if successful, otherwise Result::Err containing the std::io::Error that caused the failure.

This is the equivalent of Terminal::draw but the render callback is a function or closure that returns a Result instead of nothing.

Applications should call try_draw or draw in a loop to continuously render the terminal. These methods are the main entry points for drawing to the terminal.

This method will:

  • autoresize the terminal if necessary
  • call the render callback, passing it a Frame reference to render to
  • flush the current internal state by copying the current buffer to the backend
  • move the cursor to the last known position if it was set during the rendering closure
  • return a [CompletedFrame] with the current buffer and the area of the terminal

The render callback passed to try_draw can return any Result with an error type that can be converted into an std::io::Error using the Into trait. This makes it possible to use the ? operator to propagate errors that occur during rendering. If the render callback returns an error, the error will be returned from try_draw as an std::io::Error and the terminal will not be updated.

The [CompletedFrame] returned by this method can be useful for debugging or testing purposes, but it is often not used in regular applicationss.

The render callback should fully render the entire frame when called, including areas that are unchanged from the previous frame. This is because each frame is compared to the previous frame to determine what has changed, and only the changes are written to the terminal. If the render function does not fully render the frame, the terminal will not be in a consistent state.

Examples
use std::io;

use ratatui::widgets::Paragraph;

// with a closure
terminal.try_draw(|frame| {
    let value: u8 = "not a number".parse().map_err(io::Error::other)?;
    let area = frame.area();
    frame.render_widget(Paragraph::new("Hello World!"), area);
    frame.set_cursor_position(Position { x: 0, y: 0 });
    io::Result::Ok(())
})?;

// or with a function
terminal.try_draw(render)?;

fn render(frame: &mut ratatui::Frame) -> io::Result<()> {
    let value: u8 = "not a number".parse().map_err(io::Error::other)?;
    frame.render_widget(Paragraph::new("Hello World!"), frame.area());
    Ok(())
}
pub fn hide_cursor(&mut self) -> Result<()>

Hides the cursor.

pub fn show_cursor(&mut self) -> Result<()>

Shows the cursor.

pub fn get_cursor_position(&mut self) -> Result<Position>

Gets the current cursor position.

This is the position of the cursor after the last draw call.

pub fn set_cursor_position<P: Into<Position>>( &mut self, position: P, ) -> Result<()>

Sets the cursor position.

pub fn clear(&mut self) -> Result<()>

Clear the terminal and force a full redraw on the next draw call.

pub fn swap_buffers(&mut self)

Clears the inactive buffer and swaps it with the current buffer

pub fn size(&self) -> Result<Size>

Queries the real size of the backend.