pub struct RowBuilder { /* private fields */ }Expand description
Incrementally wraps input text into visual rows of at most width cells.
Step 1: plain-text only. ANSI-carry and styled spans will be added later.
Implementations
Sourceimpl RowBuilder
impl RowBuilder
pub fn new(target_width: usize) -> Self
pub const fn width(&self) -> usize
pub fn set_width(&mut self, width: usize)
Sourcepub fn push_fragment(&mut self, fragment: &str)
pub fn push_fragment(&mut self, fragment: &str)
Push an input fragment. May contain newlines.
Sourcepub fn end_line(&mut self)
pub fn end_line(&mut self)
Mark the end of the current logical line (equivalent to pushing a ‘\n’).
Sourcepub fn drain_rows(&mut self) -> Vec<Row>
pub fn drain_rows(&mut self) -> Vec<Row>
Drain and return all produced rows.
Sourcepub fn display_rows(&self) -> Vec<Row>
pub fn display_rows(&self) -> Vec<Row>
Rows suitable for display, including the current partial line if any.
Sourcepub fn drain_commit_ready(&mut self, max_keep: usize) -> Vec<Row>
pub fn drain_commit_ready(&mut self, max_keep: usize) -> Vec<Row>
Drain the oldest rows that exceed max_keep display rows (including the
current partial line, if any). Returns the drained rows in order.